//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// GameGenieCPU.h: interface for the GameGenieCPU class.
//////////////////////////////////////////////////////////////////////
// Class Description:  
// This class is intended as an extension for the NesCPU class, adding
// Game Genie functionality.  Had the NES CPU RAM been made a separate class
// as the PPU RAM was, it would have been extended instead.  Anyway,
// the idea is to add an extra method that adds codes, and then in the RAM_Read
// method monitor all the proper addresses and return the code data instead.
//
// All information for Game Genie decoding comes from Benzene's document
//////////////////////////////////////////////////////////////////////

#ifndef GAMEGENIE_CPU
#define GAMEGENIE_CPU

#include "NesCPU.h"


#include <vector>
using std::vector;


struct CodePatch {
	Uint16 addr;
	Uint8  data;
	Uint8  comp;
	bool   bln6char;
};


class GameGenieCPU : public NesCPU {
public:
	///////////////////////////////
	// Class Procedures
	///////////////////////////////
	GameGenieCPU() { codes.clear(); };
	virtual ~GameGenieCPU() { codes.clear(); };

	void AddCode(char *code);
	
	virtual unsigned char RAM_Read(Uint16 addr);
	virtual void executeInstruction();

private:
	CodePatch Decode6CharCode(char *code);	
	CodePatch Decode8CharCode(char *code);	
	Uint8 CharToBits(char c);

	vector<CodePatch> codes;
};

#endif
