//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// Nes Mapper 2.cpp - Implementation of mapper #2 (FFE F4xxx)
//////////////////////////////////////////////////////////////////////

#include "NesMapper006.h"


NesMapper6::NesMapper6(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *&linkPPU) {
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;

	// Replace the standard PPU memory object with a FFE F4xxx specific one
	NesPPU *newPPU = new FFEPPU();
	newPPU->Init();
	newPPU->VRAM->mbytMirroringType = linkPPU->VRAM->mbytMirroringType;
	newPPU->VRAM->link(this);	
	delete mPpu;
	mPpu = newPPU;
	linkPPU = mPpu;
	mCpu->linkCPU(mPpu, mCpu->mApu, mCpu->mController1, mCpu->mController2, this);

	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();

	blnIrqEnable = false;
	intIrqCount = 0;
}


void NesMapper6::initMemory() {	
	if (bytPRGCount > 0) {		
		swapPRGBank16(0x8000, 0);			  // First 16k bank to $8000
		//swapPRGBank16(0xC000, bytPRGCount-1); // Last 16k bank to $C000
		swapPRGBank16(0xC000, 7); // Last 16k bank to $C000		
	}
}


void NesMapper6::write(Uint16 addr, unsigned char data) {			
	// $8000 - $FFFF : Select PRGROM, CHRROM
	if (addr & 0x8000) {		
		swapPRGBank16(0x8000, ((data & 0x3C) >> 2) % bytPRGCount );
		if (bytCHRCount) swapCHRBank8( (data & 0x3) % bytCHRCount );
	} else {

		switch (addr) {	
		case 0x42FE:	// Set single screen mirroring
			mPpu->VRAM->mbytMirroringType = SINGLESCREEN;
			mPpu->VRAM->mintSSMirrorLocation = (data & 0x10) ? 0x2000 : 0x2400;
			break;

		case 0x42FF:	// Set horizontal/vertical mirroring
			mPpu->VRAM->mbytMirroringType = (data & 0x10) ? HORIZONTAL : VERTICAL;
			break;
		
		case 0x4501:
			// Disable IRQs
			blnIrqEnable = false;
			break;
		
		case 0x4502:
			// Set low byte of IRQ counter
			intIrqCount = (intIrqCount & 0xFF00) | data;
			break;

		case 0x4503:
			// Set high byte of IRQ counter, enable IRQs
			intIrqCount = (intIrqCount & 0x00FF) | (data << 8);
			blnIrqEnable = true;
			break;
		
		default:
			mCpu->memory[addr] = data;
			break;
		}	
	}
}