//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// Nes Mapper 9.cpp - Implementation of mapper #9 (MMC2)
//////////////////////////////////////////////////////////////////////

#include "NesMapper009.h"



NesMapper9::NesMapper9(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU)
{
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;

	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;

	intLatchReg1 = intLatchReg2 = 0;

	// Replace the standard PPU memory object with a MMC2 specific one	
	NesPPUMemory *newPPUMemory = new MMC2PPUMemory();
	newPPUMemory->link(this);
	newPPUMemory->init();		
	newPPUMemory->mintLatchValue1 = 0;
	newPPUMemory->mintLatchValue2 = 0;
	delete mPpu->VRAM;
	mPpu->VRAM = newPPUMemory;
	
	initMaps();
}


NesMapper9::~NesMapper9()
{
}

// Initial Memory Loader
// For mapper #9:
// The first 8K ROM bank is loaded into $8000, and the last 3 8K banks are permanently loaded
// into the last 3 slots
void NesMapper9::initMemory() {	
	if (bytPRGCount) {
		swapPRGBank8(0x8000, 0);
		swapPRGBank8(0xA000, bytPRGCount*2 - 3);
		swapPRGBank8(0xC000, bytPRGCount*2 - 2);
		swapPRGBank8(0xE000, bytPRGCount*2 - 1);
	}

	if (bytCHRCount) swapCHRBank8(bytCHRCount-1);
	
	intBankMap8 = 0;
}


void NesMapper9::write(Uint16 addr, unsigned char data) {	
	data = data & 0x1F;
	switch (addr & 0xF000) {
	
	case 0xA000: // $A000 - $AFFF: 8K PRGROM select
		if (data != intBankMap8) {
			intBankMap8 = data;
			swapPRGBank8(0x8000, intBankMap8);
		}
		break;
	
	////////////////////////
	case 0xB000: // $B000 - $BFFF: Select 4k VROM at PPU $0000 when latch is $FD						
		if (mPpu->VRAM->mintLatchSelect1 == 0xFD) {						
			if (mPpu->VRAM->mintLatchValue1 != data) swapCHRBank4(0x0000, data);
		}
		mPpu->VRAM->mintLatchValue1 = data;
		break;

	case 0xC000: // $C000 - $CFFF: Select 4k VROM at PPU $0000 when latch is $FE						
		if (mPpu->VRAM->mintLatchSelect1 == 0xFE) {			
			if (mPpu->VRAM->mintLatchValue2 != data) swapCHRBank4(0x0000, data);			
		}
		mPpu->VRAM->mintLatchValue2 = data;
		break;
	///////////////////
	
	case 0xD000: // $D000 - $DFFF: Select 4k VROM bank at PPU $1000 when latch selector is $FD								
		if (mPpu->VRAM->mintLatchSelect2 == 0xFD) {					
			if (mPpu->VRAM->mintLatchValue3 != data) swapCHRBank4(0x1000, data);			
		}
		mPpu->VRAM->mintLatchValue3 = data;
		break;
	
	case 0xE000: // $E000 - $EFFF: Select 4k VROM bank at PPU $1000 when latch selector is $FE						
		//if (data == 0x6) data = 0x7;
		if (mPpu->VRAM->mintLatchSelect2 == 0xFE) {						
			if (mPpu->VRAM->mintLatchValue4 != data) swapCHRBank4(0x1000, data);			
		}
		mPpu->VRAM->mintLatchValue4 = data;
		break;
	//////////////////////////////

	case 0xF000: // $F000 - $FFFF: Mirroring select
		if (data & 0x01) {
			// Horizontal mirroring
			mPpu->VRAM->mbytMirroringType = HORIZONTAL;			
		} else {
			// Vertical mirroring
			mPpu->VRAM->mbytMirroringType = VERTICAL;
		}		
		break;
	}
}
