//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper019.cpp - Implementation of mapper #2 (Namcot 106)
//////////////////////////////////////////////////////////////////////

#include "NesMapper019.h"


NesMapper19::NesMapper19(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *&linkPPU) {
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	// Replace the standard PPU memory object with a VRC4 specific one	
	NesPPU *newPPU = new NamcotPPU();
	newPPU->Init();
	newPPU->VRAM->mbytMirroringType = linkPPU->VRAM->mbytMirroringType;
	newPPU->VRAM->link(this);	
	delete mPpu;
	mPpu = newPPU;
	linkPPU = mPpu;
	mCpu->linkCPU(mPpu, mCpu->mApu, mCpu->mController1, mCpu->mController2, this);


	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();
	
	blnIrqEnable = false;
	intIrqCount = 0;
}


void NesMapper19::initMemory() {	
	if (bytPRGCount > 0) {
		swapPRGBank16(0x8000, 0);			  // First 16k bank to $8000
		swapPRGBank16(0xC000, bytPRGCount-1); // Last 16k bank to $C000
	}
	if (bytCHRCount) {
		swapCHRBank8(bytCHRCount-1);			// Last 8k of VROM get swapped in if present
	}
}


void NesMapper19::write(Uint16 addr, unsigned char data) {	
	switch (addr & 0xF800) {
	case 0x5000:	// Low byte of IRQ counter
		intIrqCount = (intIrqCount & 0xFF00) | data;
		break;
	case 0x5800:	// High byte of IRQ counter, IRQ enable
		intIrqCount = (intIrqCount & 0x00FF) | ((data & 0x7F) << 8);
		blnIrqEnable = (data & 0x80) ? true : false;
		break;
	
	case 0x8000:
		swapCHRBank1(0x0000, data % (bytCHRCount << 3) );
		break;
	case 0x8800:
		swapCHRBank1(0x0400, data % (bytCHRCount << 3) );
		break;	
	case 0x9000:
		swapCHRBank1(0x0800, data % (bytCHRCount << 3) );
		break;
	case 0x9800:
		swapCHRBank1(0x0C00, data % (bytCHRCount << 3) );
		break;	
	case 0xA000:
		swapCHRBank1(0x1000, data % (bytCHRCount << 3) );
		break;
	case 0xA800:
		swapCHRBank1(0x1400, data % (bytCHRCount << 3) );
		break;	
	case 0xB000:
		swapCHRBank1(0x1800, data % (bytCHRCount << 3) );
		break;
	case 0xB800:
		swapCHRBank1(0x1C00, data % (bytCHRCount << 3) );
		break;

	case 0xC000:
		if (data < 0xE0) {
			// Swap in name table data
			swapCHRBank1(0x2000, data % (bytCHRCount << 3) );
		} else {
			// Swap into VRAM isntead (?)
		}
		break;
	case 0xC800:
		if (data < 0xE0) {		
			swapCHRBank1(0x2400, data % (bytCHRCount << 3) );
		} else {
		}
		break;
	case 0xD000:
		if (data < 0xE0) {
			swapCHRBank1(0x2800, data % (bytCHRCount << 3) );
		} else {
		}
		break;
	case 0xD800:
		if (data < 0xE0) {
			swapCHRBank1(0x2C00, data % (bytCHRCount << 3) );
		} else {
		}
		break;

	case 0xE000:
		swapPRGBank8(0x8000, data % (bytPRGCount << 1));
		break;
	case 0xE800:
		swapPRGBank8(0xA000, data % (bytPRGCount << 1));
		break;
	case 0xF000:
		swapPRGBank8(0xC000, data % (bytPRGCount << 1));
		break;

	}
}