//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper22.cpp - Implementation of mapper #22 (Konami VRC2 type A)
//////////////////////////////////////////////////////////////////////

#include "NesMapper022.h"



NesMapper22::NesMapper22(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU)
{
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	for (int cbanks = 0; cbanks < 8; cbanks++) { chrbanks[cbanks] = 0; }
	initMaps();
	
	intSwitchSpace = 0x8000;
}


// We initialize the same as in mapper #21, but also swap in the first
// 8K of VROM into PPU $0000
void NesMapper22::initMemory() {
	NesMapper21::initMemory();
	if (bytCHRCount) this->swapCHRBank8(0);
}


// Keep the parts of mapper #21 that are unchanged and add the different parts
void NesMapper22::write(Uint16 addr, unsigned char data)
{	
	switch (addr) {
	case 0x8000:
	case 0x9000:
	case 0xA000:
		// These registers are handled exactly the same as in mapper 21
		NesMapper21::write(addr, data);
		break;	

	case 0xB000: // Select 1K VROM bank at PPU $0000
		swapCHRBank1(0x0000, (data >> 1) % (bytCHRCount << 3));
		break;
	case 0xB001: // Select 1K VROM bank at PPU $0400
		swapCHRBank1(0x0400, (data >> 1) % (bytCHRCount << 3));
		break;	
	case 0xC000: // Select 1K VROM bank at PPU $0800
		swapCHRBank1(0x0800, (data >> 1) % (bytCHRCount << 3));
		break;
	case 0xC001: // Select 1K VROM bank at PPU $0C00
		swapCHRBank1(0x0C00, (data >> 1) % (bytCHRCount << 3));
		break;	
	case 0xD000: // Select 1K VROM bank at PPU $1000
		swapCHRBank1(0x1000, (data >> 1) % (bytCHRCount << 3));
		break;
	case 0xD001: // Select 1K VROM bank at PPU $1400
		swapCHRBank1(0x1400, (data >> 1) % (bytCHRCount << 3));
		break;	
	case 0xE000: // Select 1K VROM bank at PPU $1800
		swapCHRBank1(0x1800, (data >> 1) % (bytCHRCount << 3));
		break;
	case 0xE001: // Select 1K VROM bank at PPU $1C00
		swapCHRBank1(0x1C00, (data >> 1) % (bytCHRCount << 3));
		break;
	}
}