//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//////////////////////////////////////////////////////////////////////
// NesMapper25.cpp - Implementation of mapper #25 (Konami VRC4 Gradius 2 type)
//////////////////////////////////////////////////////////////////////

#include "NesMapper025.h"



NesMapper25::NesMapper25(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *&linkPPU)
{
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	// Replace the standard PPU memory object with a MMC3 specific one	
	NesPPU *newPPU = new VRC4PPU();
	newPPU->Init();
	//newPPU->VRAM->mbytMirroringType = linkPPU->VRAM->mbytMirroringType;
	newPPU->VRAM->link(this);	
	newPPU->VRAM->init();	
	delete mPpu;
	mPpu = newPPU;
	linkPPU = mPpu;
	mCpu->linkCPU(mPpu, mCpu->mApu, mCpu->mController1, mCpu->mController2, this);

	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	for (int cbanks = 0; cbanks < 8; cbanks++) { chrbanks[cbanks] = 0; }
	initMaps();
	intSwitchSpace = 0x8000;
	intEnableLatch = 0;
	intIrqCount = 0;
}




void NesMapper25::write(Uint16 addr, unsigned char data)
{	
	switch (addr) {	
	case 0xF001:
		NesMapper21::write(0xF004, data);
		break;

	case 0x8000:
		if (intPrgSwap) {
			swapPRGBank8(0xC000, (data % (bytPRGCount << 1)) );
		} else {
			swapPRGBank8(0x8000, (data % (bytPRGCount << 1)) );
		}
		break;

	case 0x9001:
	case 0x9004:
		{
			int intNewSwap = data & 0x2;
			if (intNewSwap != intPrgSwap) {
				int low = PrgMap8k[2], high = PrgMap8k[0];
				swapPRGBank8(0x8000, low);
				swapPRGBank8(0xC000, high);
			}
			intPrgSwap = intNewSwap;
		}
		break;
			
	default:
		NesMapper21::write(addr, data);
		break;
	}
	

}