//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/////////////////////////////////////////////////////////////////////
// NesMapper66.cpp - Implementation of mapper #66 (GNROM)
//////////////////////////////////////////////////////////////////////

#include "NesMapper066.h"



NesMapper66::NesMapper66(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU)
{
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();
}


// When cart is started/reset, first 32k PRGROM is loaded into $8000, first 8K VROM
// into PPU $0000
void NesMapper66::initMemory()
{	
	if (bytPRGCount > 0) {
		swapPRGBank32(0);
	}
	if (bytCHRCount) {
		//swapCHRBank8(0);
	}
}


void NesMapper66::write(Uint16 addr, unsigned char data) {	
	// Selct 32k PRGROM, 8K VROM
	swapPRGBank32(((data & 0xF0) >> 4) % (bytPRGCount >> 1));	
	swapCHRBank8((data & 0x0F) % (bytCHRCount));
}