//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper069.cpp - Implementation of mapper #69 (Sunsoft FME-7)
//////////////////////////////////////////////////////////////////////

#include "NesMapper069.h"


NesMapper69::NesMapper69(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *&linkPPU) {
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;

	// Replace the standard PPU memory object with a VRC4 specific one	
	NesPPU *newPPU = new SunsoftPPU();
	newPPU->Init();
	newPPU->VRAM->mbytMirroringType = linkPPU->VRAM->mbytMirroringType;
	newPPU->VRAM->link(this);	
	delete mPpu;
	mPpu = newPPU;
	linkPPU = mPpu;
	mCpu->linkCPU(mPpu, mCpu->mApu, mCpu->mController1, mCpu->mController2, this);
	
	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();

	reg0 = 0;
	blnIrqEnable = false;
	intIrqCount = 0;
}


void NesMapper69::initMemory() {	
	if (bytPRGCount > 0) {
		swapPRGBank16(0x8000, 0);			  // First 16k bank to $8000
		swapPRGBank16(0xC000, bytPRGCount-1); // Last 16k bank to $C000
	}
	if (bytCHRCount) swapCHRBank8(0);	
}


void NesMapper69::write(Uint16 addr, unsigned char data) {	
	switch (addr) {
	case 0x8000:	// Select command
		reg0 = data & 0x0F;
		break;
	
	case 0xA000:	// Enable command
		switch (reg0) {
		case 0:
			swapCHRBank1(0x0000, data % (bytCHRCount << 3) );
			break;
		case 1:
			swapCHRBank1(0x0400, data % (bytCHRCount << 3) );
			break;
		case 2:
			swapCHRBank1(0x0800, data % (bytCHRCount << 3) );
			break;
		case 3:
			swapCHRBank1(0x0C00, data % (bytCHRCount << 3) );
			break;
		case 4:
			swapCHRBank1(0x1000, data % (bytCHRCount << 3) );
			break;
		case 5:
			swapCHRBank1(0x1400, data % (bytCHRCount << 3) );
			break;
		case 6:
			swapCHRBank1(0x1800, data % (bytCHRCount << 3) );
			break;
		case 7:
			swapCHRBank1(0x1C00, data % (bytCHRCount << 3) );
			break;
		
		case 8: // I guess this allows PRGROM to be swapped into the wRam area ($6000-$7FFF)
				// if bit 6 is not set
			if (!(data & 0x40)) {
				swapPRGBank8(0x6000, data % (bytPRGCount << 1) );
			}
			break;
		
		case 9:
			swapPRGBank8(0x8000, data % (bytPRGCount << 1) );
			break;
		case 10:
			swapPRGBank8(0xA000, data % (bytPRGCount << 1) );
			break;
		case 11:
			swapPRGBank8(0xC000, data % (bytPRGCount << 1) );
			break;

		case 12:	// Mirroring
			switch (data & 0x03) {
			case 0x00:
				mPpu->VRAM->mbytMirroringType = HORIZONTAL;
				break;
			case 0x01:			
				mPpu->VRAM->mbytMirroringType = VERTICAL;
				break;
			case 0x02:
				mPpu->VRAM->mbytMirroringType = SINGLESCREEN;
				mPpu->VRAM->mintSSMirrorLocation = 0x2000;
				break;
			case 0x03:
				mPpu->VRAM->mbytMirroringType = SINGLESCREEN;
				mPpu->VRAM->mintSSMirrorLocation = 0x2400;
				break;
			}		
			break;

		case 13:	// IRQ enable/disable
			blnIrqEnable = (data) ? true : false;
			break;
		
		case 14:	// Low byte of IRQ counter
			intIrqCount = (intIrqCount & 0xFF00) | data;
			break;
		
		case 15:	// High byte of IRQ counter
			intIrqCount = (intIrqCount & 0x00FF) | (data << 8);
			break;
		}
		break;		
	}
}