//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper079.cpp - Implementation of mapper #79 (Nina-3 AVE)
//////////////////////////////////////////////////////////////////////

#include "NesMapper079.h"


NesMapper79::NesMapper79(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU) {
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();
}


void NesMapper79::initMemory() {	
	if (bytPRGCount > 0) {
		swapPRGBank32(0);					  // First 32k bank to $8000
	}
	if (bytCHRCount) swapCHRBank8(0);		  // First 8K CHRROM to VRAM
}


void NesMapper79::write(Uint16 addr, unsigned char data) {		
	// On writes to $4100...
	if (addr == 0x4100) {
		// Bit 4 picks 32k PRGROM page
		swapPRGBank32( ((data & 0x8) >> 3) % (bytPRGCount << 1) );

		// Lower 3 bits pick 8k CHRROM page
		swapCHRBank8( (data & 0x07) % (bytCHRCount) );
	}
}