////////////////////////////////////////////////////////////////////
// Nes Mapper #7:  MMC2
// This mapper is used only on the U.S. versions of Punch-Out
////////////////////////////////////////////////////////////////////

#if !defined(NES_MAPPER_9
#define NES_MAPPER_9

#include "..\NesBaseMapper.h"
#include "..\NesPPUMemory.h"
#include <windows.h>


// Derive a special PPU memory class that maintains the swappable latch value
class MMC2PPUMemory : public NesPPUMemory
{
public:
	MMC2PPUMemory::MMC2PPUMemory() {		
		mintLatchSelect1 = 0xFD;
		mintLatchSelect2 = 0xFE;
		mintLatchSelect3 = 0xFE;
		mintLatchSelect4 = 0xFE;
		mintLatchValue1 = mintLatchValue2 = mintLatchValue3 = mintLatchValue4 = 0;
	}

	void checkLatch(int addr) {
		if (addr >= 0x1000) {
			if ( (addr & 0xFF0) == 0xFD0) {				
				if (mintLatchSelect2 != 0xFD) {
					mMapper->swapCHRBank4(0x1000, mintLatchValue3);					
				}
				mintLatchSelect2 = 0xFD;
			} else if ( (addr & 0xFF0) == 0xFE0) {				
				if (mintLatchSelect2 != 0xFE) {
					mMapper->swapCHRBank4(0x1000, mintLatchValue4);			
				}
				mintLatchSelect2 = 0xFE;
			}
		} else {
			if ( (addr & 0xFF0) == 0xFD0) {				
				if (mintLatchSelect1 != 0xFD) mMapper->swapCHRBank4(0x0000, mintLatchValue1);
				mintLatchSelect1 = 0xFD;				
			} else if ( (addr & 0xFF0) == 0xFE0) {				
				if (mintLatchSelect1 != 0xFE) mMapper->swapCHRBank4(0x0000, mintLatchValue2);				
				mintLatchSelect1 = 0xFE;			
			}
		}		
	}

	virtual byte read(unsigned __int16 &addr, bool blnCPURead) {
		// Check for latch swap, then call the super method to do standard processing
		checkLatch(addr);
		return NesPPUMemory::read(addr, blnCPURead);
	}
	
};


class NesMapper9 : public NesBaseMapper
{
public:	
	///////////////////////////////
	// Class Procedures
	///////////////////////////////
	NesMapper9(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU);
	~NesMapper9();	
	void write(unsigned __int16 addr, unsigned char data);
	void initMemory();
	
	
	///////////////////////////////
	// Class Data Members
	///////////////////////////////
	// This data member stores the 8k bank of PRG-ROM that is currently
	// loaded into memory at $8000, and is used to prevent swappnig data in that
	// is already there
	int intBankMap8, intChrLow, intChrHigh;
	int intLatchReg1, intLatchReg2;
};

#endif 
